
#include "lmsensors.h"
#include "middlelayer.h"
#include "types.h"

#include <errno.h>

#include <glib/garray.h>
#include <glib/gmessages.h>
#include <glib/gmem.h>
#include <glib/gprintf.h>
#include <glib/gstrfuncs.h>

#include <stdio.h>
#include <string.h>

/* Unused
int get_number_chip_features (const sensors_chip_name *name)
{
    int nr1 = 0, nr2 = 0, numer = 0;
    const sensors_feature_data *sfd;

    TRACE("enters get_number_chip_features");

    do {
        sfd = sensors_get_all_features (*name, &nr1, &nr2);
        if (sfd!=NULL)
            number++;
    } while (sfd!=NULL);

    TRACE("leaves get_number_chip_features");

    return number-1;
} */


t_chip *setup_chip (GPtrArray *chips, const sensors_chip_name *name, int num_sensorchips)
{
    t_chip* chip;

    TRACE("enters setup_chip");

    chip = g_new0 (t_chip, 1);

    g_ptr_array_add (chips, chip);

    chip->chip_name = (sensors_chip_name *) g_malloc (sizeof(sensors_chip_name));
    memcpy ( (void *) (chip->chip_name), (void *) name, sizeof(sensors_chip_name) );

#if SENSORS_API_VERSION < 0x400 /* libsensor 3 code */
    chip->sensorId = g_strdup_printf ("%s-%x-%x", name->prefix, name->bus, name->addr);
#else
    switch (name->bus.type) {
        case SENSORS_BUS_TYPE_I2C:
        case SENSORS_BUS_TYPE_SPI:
            chip->sensorId = g_strdup_printf ("%s-%x-%x", name->prefix,
                                              name->bus.nr, name->addr);
            break;
        default:
            chip->sensorId = g_strdup_printf ("%s-%x", name->prefix, name->addr);
    }
#endif
    chip->num_features=0;
    chip->name = g_strdup (_("LM Sensors"));
    chip->chip_features = g_ptr_array_new();

#if SENSORS_API_VERSION < 0x400 /* libsensor 3 code */
    chip->description = g_strdup (sensors_get_adapter_name (num_sensorchips-1));
#else
    chip->description = g_strdup (sensors_get_adapter_name (&name->bus));
#endif

    TRACE("leaves setup_chip");

    return chip;
}


void setup_chipfeature (t_chipfeature *chipfeature, int number, double sensorFeature)
{
    TRACE("enters setup_chipfeature");

    chipfeature->color = "#00B000";
    chipfeature->valid = TRUE;
    chipfeature->formatted_value = g_strdup_printf("%+5.1f", sensorFeature);
    chipfeature->raw_value = sensorFeature;
    chipfeature->address = number;
    chipfeature->show = FALSE;

#if SENSORS_API_VERSION < 0x400 /* libsensor 3 code */
    categorize_sensor_type (chipfeature);
#endif

    TRACE("leaves setup_chipfeature");
}


#if SENSORS_API_VERSION < 0x400 /* libsensor 3 code */
t_chipfeature *find_chipfeature    (const sensors_chip_name *name, t_chip *chip, int number)
{
#else
t_chipfeature *find_chipfeature    (const sensors_chip_name *name, t_chip *chip, const sensors_feature *feature)
{
    const sensors_subfeature *sub_feature = NULL;
    int number = -1;
#endif
    int res;
    double sensorFeature;
    t_chipfeature *chipfeature;

    TRACE("enters find_chipfeature");

    chipfeature = g_new0 (t_chipfeature, 1);

#if SENSORS_API_VERSION < 0x400 /* libsensor 3 code */
    if (sensors_get_ignored (*(name), number)==1) {
        res = sensors_get_label(*(name), number,
                                &(chipfeature->name));

        if (res==0) {
            res = sensors_get_feature (*(name), number,
                                        &sensorFeature);

#else /* libsensors 4 code */

    switch (feature->type) {
    case SENSORS_FEATURE_IN:
        sub_feature = sensors_get_subfeature(name, feature,
                                             SENSORS_SUBFEATURE_IN_INPUT);
        if (!sub_feature)
            break;

        number = sub_feature->number;
        chipfeature->class = VOLTAGE;
        chipfeature->min_value = 2.8;
        chipfeature->max_value = 12.2;

        if ((sub_feature = sensors_get_subfeature(name, feature,
                SENSORS_SUBFEATURE_IN_MIN)) &&
                !sensors_get_value(name, sub_feature->number, &sensorFeature))
            chipfeature->min_value = sensorFeature;

        if ((sub_feature = sensors_get_subfeature(name, feature,
                SENSORS_SUBFEATURE_IN_MAX)) &&
                !sensors_get_value(name, sub_feature->number, &sensorFeature))
            chipfeature->max_value = sensorFeature;

        break;

    case SENSORS_FEATURE_FAN:
        sub_feature = sensors_get_subfeature(name, feature,
                                             SENSORS_SUBFEATURE_FAN_INPUT);
        if (!sub_feature)
            break;

        number = sub_feature->number;
        chipfeature->class = SPEED;
        chipfeature->min_value = 1000.0;
        chipfeature->max_value = 3500.0;

        if ((sub_feature = sensors_get_subfeature(name, feature,
                SENSORS_SUBFEATURE_FAN_MIN)) &&
                !sensors_get_value(name, sub_feature->number, &sensorFeature))
            chipfeature->min_value = sensorFeature;

        break;

    case SENSORS_FEATURE_TEMP:
        sub_feature = sensors_get_subfeature(name, feature,
                                             SENSORS_SUBFEATURE_TEMP_INPUT);
        if (!sub_feature)
            break;

        number = sub_feature->number;
        chipfeature->class = TEMPERATURE;
        chipfeature->min_value = 0.0;
        chipfeature->max_value = 80.0;

        if ((sub_feature = sensors_get_subfeature(name, feature,
                SENSORS_SUBFEATURE_TEMP_MIN)) &&
                !sensors_get_value(name, sub_feature->number, &sensorFeature))
            chipfeature->min_value = sensorFeature;

        if (((sub_feature = sensors_get_subfeature(name, feature,
                SENSORS_SUBFEATURE_TEMP_MAX)) ||
                (sub_feature = sensors_get_subfeature(name, feature,
                SENSORS_SUBFEATURE_TEMP_CRIT))) &&
                !sensors_get_value(name, sub_feature->number, &sensorFeature))
            chipfeature->max_value = sensorFeature;
    }

    if (number != -1) {

        chipfeature->name = sensors_get_label(name, feature);

        if (!chipfeature->name)
            chipfeature->name = feature->name;

        if (chipfeature->name) {
            res = sensors_get_value (name, number,
                                        &sensorFeature);

#endif /* libsensor3 / libsensors4 code */

            if (res==0) {
                setup_chipfeature (chipfeature, number, sensorFeature);
                chip->num_features++;
                TRACE("leaves find_chipfeature");
                return chipfeature;
            }
        }
    }
    g_free(chipfeature);

    TRACE("leaves find_chipfeature with null");
    return NULL;
}


int initialize_libsensors (GPtrArray *chips)
{
    int sensorsInit, nr1, nr2, num_sensorchips; /*    , numchips;  */
    t_chip *chip;
    t_chipfeature *chipfeature; /* , *furtherchipfeature; */
    const sensors_chip_name *detected_chip;
#if SENSORS_API_VERSION < 0x400 /* libsensor 3 code */
    FILE *file;
    const sensors_feature_data *sfd;

    TRACE("enters initialize_libsensors");

    errno = 0;
    file = fopen("/etc/sensors.conf", "r");

    if (errno != ENOENT) /* the file actually exists */
    {
        sensorsInit = sensors_init(file);
        if (sensorsInit != 0)
        {
            g_printf(_("Error: Could not connect to sensors!"));
            /* FIXME: better popup window? write to special logfile? */
            return -2;
        }

        num_sensorchips = 0;
        detected_chip = sensors_get_detected_chips ( &num_sensorchips);

        /* iterate over chips on mainboard */
        while (detected_chip!=NULL)
        {
            chip = setup_chip (chips, detected_chip, num_sensorchips);

            nr1 = 0;
            nr2 = 0;
            /* iterate over chip features, i.e. id, cpu temp, mb temp... */
            /* numchips = get_number_chip_features (detected_chip); */
            sfd = sensors_get_all_features (*detected_chip, &nr1, &nr2);
            while (sfd != NULL)
            {
                chipfeature = find_chipfeature (detected_chip, chip, sfd->number);
                if (chipfeature!=NULL) {
                    g_ptr_array_add (chip->chip_features, chipfeature);
                }
                sfd = sensors_get_all_features (*detected_chip, &nr1, &nr2);
            }

            detected_chip = sensors_get_detected_chips (&num_sensorchips);
        } /* end while sensor chipNames */

        fclose (file);
        TRACE ("leaves initialize_libsensors with 1");
        return 1;
    }
    else {
        fclose (file);
        TRACE ("leaves initialize_libsensors with -1");
        return -1;
    }
#else
    const sensors_feature *sfd;
    TRACE("enters initialize_libsensors");
    
    sensorsInit = sensors_init(NULL);
    if (sensorsInit != 0)
    {
        g_printf(_("Error: Could not connect to sensors!"));
        /* FIXME: better popup window? write to special logfile? */
        return -2;
    }

    num_sensorchips = 0;
    detected_chip = sensors_get_detected_chips (NULL, &num_sensorchips);
    /* iterate over chips on mainboard */
    while (detected_chip!=NULL)
    {
        chip = setup_chip (chips, detected_chip, num_sensorchips);

        nr1 = 0;
        nr2 = 0;
        /* iterate over chip features, i.e. id, cpu temp, mb temp... */
        /* numchips = get_number_chip_features (detected_chip); */
        sfd = sensors_get_features (detected_chip, &nr1);
        while (sfd != NULL)
        {
            chipfeature = find_chipfeature (detected_chip, chip, sfd);
            if (chipfeature!=NULL) {
                g_ptr_array_add (chip->chip_features, chipfeature);
            }
            sfd = sensors_get_features (detected_chip, &nr1);
        }

        detected_chip = sensors_get_detected_chips (NULL, &num_sensorchips);
    } /* end while sensor chipNames */

    TRACE ("leaves initialize_libsensors with 1");
    return 1;
#endif
}


void
refresh_lmsensors (gpointer chip_feature, gpointer data)
{
    t_chipfeature *cf;

    TRACE ("leaves refresh_lmsensors");

    g_assert(chip_feature!=NULL);

    cf = (t_chipfeature *) chip_feature;

    TRACE ("leaves refresh_lmsensors");
}
